import 'dart:convert';
import 'dart:typed_data';
import 'package:robomotion_app/robomotion_app.dart';
import 'package:version/version.dart';

class QueueMessage {
  String? id;
  String? appID;
  String? robotID;
  String? userID;
  int? msgType;
  Uint8List? payload;
  bool? encrypted;
  Version? version;
  Robomotion? robomotion;

  QueueMessage({
    required this.id,
    required this.appID,
    this.robotID,
    this.userID,
    this.msgType = QMT_KeyExchange,
    this.payload,
    this.encrypted,
    this.version,
  });

  factory QueueMessage.fromJson(Map<dynamic, dynamic> json) => QueueMessage(
        id: json["id"],
        appID: json["app_id"],
        robotID: json["robot_id"],
        userID: json["user_id"],
        msgType: json["msg_type"],
        payload: base64Decode(json["payload"]),
        encrypted: json["encrypted"],
        version:
            json["version"] == null ? null : Version.parse(json["version"]),
      );

  Map<String, dynamic> toJson() => {
        "id": id,
        "app_id": appID,
        "robot_id": robotID,
        "user_id": userID,
        "msg_type": msgType,
        "payload": payload,
        "encrypted": encrypted,
        "version": version?.toString(),
      };

  @override
  String toString() {
    return jsonEncode(this);
  }

  void reply(Uint8List payload) {
    if (this.msgType != QMT_AppRequest) {
      throw Exception("Cannot reply to a non-request message");
    }

    if (this.robomotion == null) {
      throw Exception("Cannot reply to a message without robomotion instance");
    }

    this.payload = payload;
    this.robomotion!.sendAppMsg(this);
  }
}

const int QMT_Error = -1;
const int QMT_KeyExchange = 0;
const int QMT_AppIn = 1;
const int QMT_AppOut = 2;
const int QMT_AppPush = 3;
const int QMT_AppRequest = 4;
const int QMT_RobotID = 5;
